/**
 * Returns an array of values between the min and max values.
 *
 * ```typescript
 * import { rangeArr } from 'goodish'
 * const arr = rangeArr(0, 5) // [0, 1, 2, 3, 4, 5]
 * ```
 *
 * @param min
 * @param max
 * @param step
 */
export declare function rangeArr(min: number, max: number, step?: number): number[];
/**
 * A generator which yields each value between the min and max values
 *
 * ```typescript
 * import { range } from 'goodish'
 * for (const n of range(0, 5)) {
 *   console.log(n) // 0, 1, 2, 3, 4
 * }
 * ```
 * @param min
 * @param max
 * @param step
 */
export declare function range(min: number, max: number, step?: number): IterableIterator<number>;
/**
 * Count the number of items in an iterator
 *
 * ```typescript
 * import { count } from 'goodish'
 * console.log(count([0, 1, 2, 3, 4])) // 5
 * ```
 *
 * @param arr
 */
export declare function count(arr: Iterable<any>): number;
/**
 * Returns a tuple containing a random element from a collection and the index of that element in the collection.
 *
 * ```typescript
 * import { randomFrom, setSeed } from 'goodish'
 * const arr = [0, 1, 2, 3, 4]
 * console.log(randomFrom(arr)) // any single element of arr. will be different each time this code runs
 * setSeed(1)
 * console.log(randomFrom(arr)) // always the same element of arr
 * ```
 * @param arr
 */
export declare function randomFrom<T>(arr: T[]): T;
/**
 * Shuffle an array in place.
 *
 * ```typescript
 * import { shuffle, setSeed } from 'goodish'
 * const arr = [0, 1, 2, 3, 4]
 * console.log(shuffle(arr.slice())) // a different array order each time this code is run
 * setSeed(1)
 * console.log(shuffle(arr.slice()) // a different array order, but will be the same each time the code is run
 * ```
 *
 * @param arr
 * @reference Adapted from [here](https://stackoverflow.com/a/6274381/5551941)
 */
export declare function shuffle<T>(arr: T[]): T[];
/**
 * A generator which yields all of the permutations of an array
 *
 * ```typescript
 * import { permutationsOf } from 'goodish'
 * console.log(Array.from(permutationsOf([0, 1, 2, 3, 4]))
 * ```
 *
 * @param arr
 * @reference Adapted from [this answer](https://stackoverflow.com/a/37580979/5551941)
 */
export declare function permutationsOf<T>(arr: T[]): IterableIterator<T[]>;
/**
 * A generator which yields all of the combinations of N items from an array.
 *
 * ```typescript
 * import { combinationsOf } from 'goodish'
 * console.log(Array.from(combinationsOf([0, 1, 2], 2)) // [[0, 1], [0, 2], [1, 2]]
 * ```
 *
 * @param arr
 * @param n
 * @reference Copied verbatim from [this site](https://lowrey.me/es6-javascript-combination-generator/)
 */
export declare function combinationsOf<T>(arr: T[], n: number): IterableIterator<T[]>;
/**
 * Swap two indexes of an array
 *
 * ```typescript
 * import { swap } from 'goodish'
 * const arr = [0, 1, 2]
 * swap(1, 2)
 * console.log(arr) // [0, 2, 1]
 * ```
 *
 * @param arr
 * @param indexA
 * @param indexB
 */
export declare function swap<T>(arr: T[], indexA: number, indexB: number): T[];
/**
 * Transpose a 2D array.
 *
 * ```typescript
 * import { transpose } from 'goodish'
 * console.log(transpose([[0], [1], [2]])) // [[0, 1, 2]]
 * ```
 *
 * @param arr
 * @reference Copied verbatim from [this answer](https://stackoverflow.com/a/17428705/5551941)
 */
export declare function transpose<T>(arr: T[][]): T[][];
