"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Math_1 = require("./Math");
/**
 * Returns an array of values between the min and max values.
 *
 * ```typescript
 * import { rangeArr } from 'goodish'
 * const arr = rangeArr(0, 5) // [0, 1, 2, 3, 4, 5]
 * ```
 *
 * @param min
 * @param max
 * @param step
 */
function rangeArr(min, max, step = 1) {
    return Array.from(range(min, max, step));
}
exports.rangeArr = rangeArr;
/**
 * A generator which yields each value between the min and max values
 *
 * ```typescript
 * import { range } from 'goodish'
 * for (const n of range(0, 5)) {
 *   console.log(n) // 0, 1, 2, 3, 4
 * }
 * ```
 * @param min
 * @param max
 * @param step
 */
function* range(min, max, step = 1) {
    for (let i = min; i < max; i += step) {
        yield i;
    }
}
exports.range = range;
/**
 * Count the number of items in an iterator
 *
 * ```typescript
 * import { count } from 'goodish'
 * console.log(count([0, 1, 2, 3, 4])) // 5
 * ```
 *
 * @param arr
 */
function count(arr) {
    let c = 0;
    for (const _ of arr) {
        c++;
    }
    return c;
}
exports.count = count;
/**
 * Returns a tuple containing a random element from a collection and the index of that element in the collection.
 *
 * ```typescript
 * import { randomFrom, setSeed } from 'goodish'
 * const arr = [0, 1, 2, 3, 4]
 * console.log(randomFrom(arr)) // any single element of arr. will be different each time this code runs
 * setSeed(1)
 * console.log(randomFrom(arr)) // always the same element of arr
 * ```
 * @param arr
 */
function randomFrom(arr) {
    if (!arr.length)
        return null;
    return arr[Math_1.randomInt(0, arr.length - 1)];
}
exports.randomFrom = randomFrom;
/**
 * Shuffle an array in place.
 *
 * ```typescript
 * import { shuffle, setSeed } from 'goodish'
 * const arr = [0, 1, 2, 3, 4]
 * console.log(shuffle(arr.slice())) // a different array order each time this code is run
 * setSeed(1)
 * console.log(shuffle(arr.slice()) // a different array order, but will be the same each time the code is run
 * ```
 *
 * @param arr
 * @reference Adapted from [here](https://stackoverflow.com/a/6274381/5551941)
 */
function shuffle(arr) {
    let m = arr.length;
    let t;
    let i;
    while (m) {
        i = Math.floor(Math_1.random() * m--);
        t = arr[m];
        arr[m] = arr[i];
        arr[i] = t;
    }
    return arr;
}
exports.shuffle = shuffle;
/**
 * A generator which yields all of the permutations of an array
 *
 * ```typescript
 * import { permutationsOf } from 'goodish'
 * console.log(Array.from(permutationsOf([0, 1, 2, 3, 4]))
 * ```
 *
 * @param arr
 * @reference Adapted from [this answer](https://stackoverflow.com/a/37580979/5551941)
 */
function* permutationsOf(arr) {
    const permutation = arr.slice(0);
    const c = Array(arr.length).fill(0);
    let i = 1;
    let k;
    let p;
    yield permutation.slice();
    while (i < arr.length) {
        if (c[i] < i) {
            k = i % 2 && c[i];
            p = permutation[i];
            permutation[i] = permutation[k];
            permutation[k] = p;
            c[i]++;
            i = 1;
            yield permutation.slice();
        }
        else {
            c[i] = 0;
            i++;
        }
    }
}
exports.permutationsOf = permutationsOf;
/**
 * A generator which yields all of the combinations of N items from an array.
 *
 * ```typescript
 * import { combinationsOf } from 'goodish'
 * console.log(Array.from(combinationsOf([0, 1, 2], 2)) // [[0, 1], [0, 2], [1, 2]]
 * ```
 *
 * @param arr
 * @param n
 * @reference Copied verbatim from [this site](https://lowrey.me/es6-javascript-combination-generator/)
 */
function* combinationsOf(arr, n) {
    for (let i = 0; i < arr.length; i++) {
        if (n === 1) {
            yield [arr[i]];
        }
        else {
            let remaining = combinationsOf(arr.slice(i + 1, arr.length), n - 1);
            for (let next of remaining) {
                yield [arr[i], ...next];
            }
        }
    }
}
exports.combinationsOf = combinationsOf;
/**
 * Swap two indexes of an array
 *
 * ```typescript
 * import { swap } from 'goodish'
 * const arr = [0, 1, 2]
 * swap(1, 2)
 * console.log(arr) // [0, 2, 1]
 * ```
 *
 * @param arr
 * @param indexA
 * @param indexB
 */
function swap(arr, indexA, indexB) {
    const v = arr[indexA];
    arr[indexA] = arr[indexB];
    arr[indexB] = v;
    return arr;
}
exports.swap = swap;
/**
 * Transpose a 2D array.
 *
 * ```typescript
 * import { transpose } from 'goodish'
 * console.log(transpose([[0], [1], [2]])) // [[0, 1, 2]]
 * ```
 *
 * @param arr
 * @reference Copied verbatim from [this answer](https://stackoverflow.com/a/17428705/5551941)
 */
function transpose(arr) {
    return arr[0].map((col, i) => arr.map(row => row[i]));
}
exports.transpose = transpose;
